from django.core.management.base import BaseCommand, CommandError
from backend import xsanio_backend

class Command(BaseCommand):
    help = 'Runs the server backend process'

    def handle(self, *args, **options):
        xsanio_backend.main()


from django.contrib import admin
from mainapp.models import *

admin.site.register(stat_type)
admin.site.register(xsanio_client)
admin.site.register(stat_entry)
admin.site.register(cvlabel_entry)
admin.site.register(xsan_volume)
admin.site.register(xsan_volume_lun)



from django.views.generic import ListView
from django.views.generic import DetailView
from mainapp import models


class display_client_list(ListView):
    template_name = 'client_list.html'
    model = models.xsanio_client


class display_client_detail(DetailView):
    template_name = 'client_detail.html'
    model = models.xsanio_client

    def get_context_data(self, **kwargs):
        # Call the base implementation first to get a context
        context = super(display_client_detail, self).get_context_data(**kwargs)

        disks_iodata = {}

        client = models.xsanio_client.objects.get(pk=self.kwargs['pk'])
        client_stat_entries = models.stat_entry.objects.filter(
            host=client,
        )

        for client_stat_entry in client_stat_entries:
            disk_name = client_stat_entry.disk

            if disk_name not in disks_iodata:
                disks_iodata[disk_name] = {}

            disk_iodata = disks_iodata[disk_name]
            disk_iodata[client_stat_entry.type.name] = client_stat_entry.value

        disks_iodata_array = []
        for disk_name in disks_iodata:
            result = {}
            #   Is there a cvlabel for this disk?
            disk_cvlabel = models.cvlabel_entry.objects.filter(
                host=client,
                disk=disk_name,
            ).all()

            if disk_cvlabel:
                result['disk_name'] = disk_cvlabel[0].label
            else:
                result['disk_name'] = disk_name

            for stat_type in models.stat_type.objects.all():
                result[stat_type.name] = disks_iodata[disk_name][stat_type.name]

            disks_iodata_array.append(result)

        #   Sorting!
        if 'stat_type_order' in self.kwargs:
            stat_type_order = self.kwargs['stat_type_order']
            disks_iodata_array = reversed(sorted(disks_iodata_array, key=lambda k: k[stat_type_order]))

        global_stats = {}
        for stat_type in models.stat_type.objects.all():
            stat_name = stat_type.name
            for disk_name in disks_iodata:
                if stat_name not in global_stats:
                    global_stats[stat_name] = disks_iodata[disk_name][stat_name]
                else:
                    global_stats[stat_name] += disks_iodata[disk_name][stat_name]

        context['disks_iodata'] = disks_iodata_array
        context['global_stats'] = global_stats
        return context



from django.db import models


class xsanio_client(models.Model):
    """Can be a client, can be an MDC -- the only thing that matters
    is xsanio_client being installed on the
    machine

    """
    host_name = models.CharField(max_length=255)
    ip_address = models.IPAddressField()
    port = models.IntegerField(default=8090)
    is_unreachable = models.BooleanField(default=True)
    has_client_installed = models.BooleanField(default=False)
    latest_update = models.DateTimeField(null=True, blank=True)

    def __unicode__(self):
        return self.host_name


class stat_type(models.Model):
    """Types of statistics we have. E.g. I/O per second, bytes read per second and so on.

    """
    name = models.CharField(max_length=255)

    def __unicode__(self):
        return self.name

class stat_entry(models.Model):
    """Stat entry is a specific value of a specific task type for a specific host

    """
    type = models.ForeignKey(stat_type)
    host = models.ForeignKey(xsanio_client)
    disk = models.CharField(max_length=255)
    value = models.FloatField(null=True, blank=True, default=0.0)
    time_stamp = models.DateTimeField(auto_now_add=True)

    def __unicode__(self):
        return "%s: %s for %s" % (self.host, self.type, self.disk)

class cvlabel_entry(models.Model):
    """Stores one 'disk name' -- 'label name' relationship for a specific host

    """
    host = models.ForeignKey(xsanio_client)
    disk = models.CharField(max_length=255)
    label = models.CharField(max_length=255)

    def __unicode__(self):
        return "%s: %s " % (self.host, self.disk)

class xsan_volume(models.Model):
    """We need to remember, which LUNs each volume is composed of

    """
    name = models.CharField(max_length=255)

    def __unicode__(self):
        return self.name

class xsan_volume_lun(models.Model):
    """An entry for each LUN that is assigned to particular Xsan volume

    """
    label = models.CharField(max_length=255)
    volume = models.ForeignKey(xsan_volume)

    def __unicode__(self):
        return self.label



from django.conf.urls import patterns, include, url
from mainapp.views import volume
from mainapp.views import client

# Uncomment the next two lines to enable the admin:
from django.contrib import admin
admin.autodiscover()

urlpatterns = patterns('',
    # Examples:
    # url(r'^$', 'xsanio_server.views.home', name='home'),
    # url(r'^xsanio_server/', include('xsanio_server.foo.urls')),

    # Uncomment the admin/doc line below to enable admin documentation:
    # url(r'^admin/doc/', include('django.contrib.admindocs.urls')),

    # Uncomment the next line to enable the admin:
    url(r'^admin/', include(admin.site.urls)),
)

urlpatterns += patterns('django.views.generic.simple',
    url(r'^$', 'direct_to_template', {'template': 'home.html'}, name='home_view'),
)

urlpatterns += patterns('',
    url(
        r'^volumes/$',
        volume.display_volume_list.as_view(),
        name='volume_list_view'
    ),
    url(
        r'^volumes/(?P<pk>\d+)/$',
        volume.display_volume_detail.as_view(),
        name='volume_detail_view'
    ),
    url(
        r'^volumes/(?P<pk>\d+)/order_by/(?P<stat_type_order>.+)/$',
        volume.display_volume_detail.as_view(),
        name='volume_detail_view_sorted'
    ),
    url(
        r'^clients/$',
        client.display_client_list.as_view(),
        name='client_list_view'
    ),
    url(
        r'clients/(?P<pk>\d+)/$',
        client.display_client_detail.as_view(),
        name='client_detail_view'
    ),
    url(
        r'clients/(?P<pk>\d+)/order_by/(?P<stat_type_order>.+)/$',
        client.display_client_detail.as_view(),
        name='client_detail_view_sorted'
    )
)